/*
-----------------------------------------------------------------------------
This source file is part of OGRE
(Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2005 The OGRE Team
Also see acknowledgements in Readme.html

You may use this sample code for anything you like, it is not covered by the
LGPL like the rest of the engine.
-----------------------------------------------------------------------------
*/
/*
-----------------------------------------------------------------------------
Filename:    ExampleApplication.h
Description: Base class for all the OGRE examples
-----------------------------------------------------------------------------
*/

#ifndef __ExampleApplication_H__
#define __ExampleApplication_H__


class ExampleFrameListener;
namespace Ogre
{
	class Camera;
	class RenderWindow;
	class Root;
	class SceneManager;
	class Viewport;
}


/** Base class which manages the standard startup of an Ogre application.
    Designed to be subclassed for specific examples if required.
*/
class ExampleApplication
{
public:
    ExampleApplication();
    virtual ~ExampleApplication();

    /// Start the example
    virtual void go();
protected:
    Ogre::Root* mRoot;
    Ogre::Camera* mCamera;
    Ogre::SceneManager* mSceneMgr;
    ExampleFrameListener* mFrameListener;
	Ogre::Viewport* mViewport;
    Ogre::RenderWindow* mWindow;

    // These internal methods package up the stages in the startup process
    /** Sets up the application - returns false if the user chooses to abandon configuration. */
    virtual bool setup();

    /** Configures the application - returns false if the user chooses to abandon configuration. */
    virtual bool configure();

    virtual void chooseSceneManager();

    virtual void createCamera();

    virtual void createFrameListener() = 0;

    virtual void createScene() = 0;    // pure virtual - this has to be overridden

    virtual void destroyScene();    // Optional to override this

    virtual void createViewports();

    /// Method which will define the source of resources (other than current folder)
    virtual void setupResources();

	/// Optional override method where you can create resource listeners (e.g. for loading screens)
	virtual void createResourceListener();

	/// Optional override method where you can perform resource group loading
	/// Must at least do ResourceGroupManager::getSingleton().initialiseAllResourceGroups();
	virtual void loadResources();
};


#endif