#ifndef MESHPARTICLE_H
#define MESHPARTICLE_H


class Renderer;

namespace Ogre
{
	class Entity;
	class Material;
	class SceneNode;
}
#include "OgreVector3.h"


class MeshParticle
{
public:
	MeshParticle(Ogre::SceneNode* root,
				 const Ogre::Vector3& position,
				 const Ogre::Vector3& speed,
				 float lifeTime,
				 float scale,
				 const std::string& mesh,
				 const std::string& materialName,
				 bool materialRequiresDepthTexture,
				 const std::string& textureName,
				 const std::string& thicknessTextureName,
				 float volumeDensity,
				 bool animateParameters,
				 Renderer* renderer);
	~MeshParticle();

	void update(float time, const Ogre::Vector3& viewPosition);
	bool wantsToBeDestroyed() const;

	void setVolumeMode(const std::string& mesh,
					   const std::string& material,
					   bool materialRequiresDepthTexture);

	void setVolumeDensity(float density);
private:
	void removeVisualisation();
	void loadVisualisation();
	void setEntityParameters();

	Renderer* mRenderer;

	Ogre::Entity* mEntity;
	Ogre::SceneNode* mNode;
	Ogre::Material* mMaterial;

	Ogre::Vector3 mSpeed;
	float mLifeTime, mTimeExisting;
	float mInitialScale, mCurrentScale;
	std::string mMesh, mMaterialName;
	std::string mTextureName, mThicknessTextureName;
	bool mMaterialRequiresDepthTexture;
	float mVolumeDensity;
	bool mAnimateParameters;

	std::string mID;
};


#endif