#ifndef RENDERER_H
#define RENDERER_H


namespace Ogre
{
	class Camera;
	class Overlay;
	enum PixelFormat;
	class RenderTarget;
	class SceneManager;
	class SceneNode;
	class Texture;
	class Viewport;
}
#include "OgreRenderTargetListener.h"
#include <string>
#include <vector>


class Renderer: public Ogre::RenderTargetListener
{
private:
	struct RenderTexture
	{
		Ogre::Texture* texture;
		Ogre::RenderTarget* renderTarget;
		Ogre::Viewport* viewport;
	};
public:
	Renderer(Ogre::Viewport* screenViewport,
			 Ogre::RenderWindow* renderWindow,
			 Ogre::SceneManager* sceneManager,
			 Ogre::SceneNode* particleRootNode,
			 Ogre::Overlay* hudOverlay);
	virtual ~Renderer();
	virtual void preViewportUpdate(const Ogre::RenderTargetViewportEvent& evt);
	virtual void postViewportUpdate(const Ogre::RenderTargetViewportEvent& evt);

	void addMaterialWithDepth(const std::string& material);
	void setActive(bool active);
	void setRenderColourFloatOverlay(bool renderColourFloatOverlay);
private:
	RenderTexture* createRenderTexture(unsigned int width, unsigned int height,
									   Ogre::PixelFormat format);

	static void destroyRenderTexture(RenderTexture& renderTexture);

	static void addTextureToMaterial(const RenderTexture& renderTexture,
									 const std::string materialName);

	void handleDepthTextureRendering();
	void handleSceneToFloatBufferRendering();
	void detachAllObjects();
	void reattachAllObjects();

	Ogre::Viewport* mScreenViewport;
	Ogre::RenderWindow* mRenderWindow;
	Ogre::SceneManager* mSceneManager;
	Ogre::SceneNode* mParticleRootNode;

	RenderTexture mDepthTexture;
	RenderTexture mColourFloatTexture;

	Ogre::Overlay* mDebugOverlay;
	Ogre::Overlay* mHudOverlay;
	Ogre::Overlay* mColourFloatOverlay;

	bool mIsActive;
	bool mRenderColourFloatOverlay;

	std::vector<Ogre::Node*> mObjectsToReattach;
};


#endif