#include "Tools.h"

#include "OgreVector3.h"


unsigned int Tools::getUniqueNumber()
{
	static int current = 0;
	current++;
	return current;
}


std::string Tools::getUniqueString()
{
	std::stringstream numberStream("");
	numberStream << getUniqueNumber();
	return numberStream.str();
}


float Tools::getRandom(float min, float max)
{
	return float(rand()) / RAND_MAX * (max - min) + min;
}


Ogre::Vector3 Tools::rotateAroundAxis(const Ogre::Vector3& v, const Ogre::Radian& angle,
									  const Ogre::Vector3& axis)
{
	//Formula taken from
	//http://www.gamedev.net/reference/articles/article1199.asp
	float c, s, t, x, y, z, m11, m12, m13, m21, m22, m23, m31, m32, m33;
	c = std::cos(angle.valueRadians());
	s = std::sin(angle.valueRadians());
	t = 1 - c;
	x = axis.x;
	y = axis.y;
	z = axis.z;
	m11 = t * x * x + c;
	m12 = t * x * y + s * z;
	m13 = t * x * z - s * y;
	m21 = t * x * y - s * z;
	m22 = t * y * y + c;
	m23 = t * y * z + s * x;
	m31 = t * x * z + s * y;
	m32 = t * y * z - s * x;
	m33 = t * z * z + c;
	Ogre::Vector3 result = Ogre::Vector3(0, 0, 0);
	result.x = m11 * v.x + m12 * v.y + m13 * v.z;
	result.y = m21 * v.x + m22 * v.y + m23 * v.z;
	result.z = m31 * v.x + m32 * v.y + m33 * v.z;
	return result;
}


int Tools::round(float value)
{
	int result = value;
	if (value - result > 0.5f)
	{
		result++;
	}
	return result;
}


float Tools::round(float value, unsigned int decimals)
{
	int multiplier = pow(10, decimals);
	int rounded = multiplier * value;
	return float(rounded) / multiplier;
}


std::string Tools::toUpperCase(std::string text)
{
	int diff = 'A' - 'a';
	size_t size = text.size();
	for (size_t i = 0; i < size; ++i)
	{
		if (text[i] >= 'a' && text[i] <= 'z')
		{
			text[i] += diff;
		}
	}
	return text;
}