#ifndef TOOLS_H
#define TOOLS_H


namespace Ogre
{
	class Radian;
	class Vector3;
}
#include <string>


/**
* Contains generally useful static functions.
*/
class Tools
{
public:
	static unsigned int getUniqueNumber();
	static std::string getUniqueString();

	static float getRandom(float min, float max);

	//rotates a vector "v" by "rotation" radians around the axis in "axis"
	static Ogre::Vector3 rotateAroundAxis(const Ogre::Vector3& v,
										  const Ogre::Radian& angle,
										  const Ogre::Vector3& axis);
	
	//rounds a float, so 2.5 becomes 3 and 2.49 becomes 2
	static int round(float value);

	//rounds a float to the chosen number of decimals
	static float round(float value, unsigned int decimals);

	static std::string toUpperCase(std::string text);
	
	//appends two things into a string
	template <typename T1, typename T2>
	static std::string append(const T1 part1, const T2 part2);

	//appends three things into a string
	template <typename T1, typename T2, typename T3>
	static std::string append(const T1 part1, const T2 part2, const T3 part3);

	//appends four things into a string
	template <typename T1, typename T2, typename T3, typename T4>
	static std::string append(const T1 part1, const T2 part2, const T3 part3, const T4 part4);

	//turns the content into a string
	template <typename T>
	static std::string makeString(const T content);
};


template <typename T1, typename T2>
static std::string Tools::append(const T1 part1, const T2 part2)
{
	std::stringstream result;
	result << part1 << part2;
	return result.str();
}


template <typename T1, typename T2, typename T3>
static std::string Tools::append(const T1 part1, const T2 part2, const T3 part3)
{
	std::stringstream result;
	result << part1 << part2 << part3;
	return result.str();
}

template <typename T1, typename T2, typename T3, typename T4>
static std::string Tools::append(const T1 part1, const T2 part2, const T3 part3, const T4 part4)
{
	std::stringstream result;
	result << part1 << part2 << part3 << part4;
	return result.str();
}


template <typename T>
static std::string Tools::makeString(const T content)
{
	std::stringstream result;
	result << content;
	return result.str();
}


#endif